;=====================================================
; Specter Knight-Style Walljump and Wallclimb
; By Erik557
;=====================================================
; This patch is based on Specter Knight's movement as
; seen on his campaign in Shovel Knight: Specter of
; Torment. I made it more customizable though!
;=====================================================

;   Some defines and SA-1 check
;   Free RAM addresses that should be in the $0100-$1FFF area. (unless you know what you're doing)

!RAM_TimeInWall      = $1864       ;   time the player has been in walls
!RAM_77Backup        = $1696       ;   copy of $77, last frame
!RAM_KickFlags       = $1E01       ;   copy of $77, same frame; also tells if wallkicked
!RAM_DisableMovement = $18A6       ;   has the time in which movement will be disabled

;   Other defines

!TimeToStick  = $48  ;   how much time can the player be climbing a wall.
!ClimbSpeed   = $F0  ;   speed of climbing. $FF is the slowest, $80 is the fastest. don't set to $00-$7F.
!ElevateSpeed = $C0  ;   speed at which mario will elevate after getting to the edge of a wall. same as above, but set to $00 to disable.
!KickXSpeed   = $20  ;   Wall kick X speed.
!KickYSpeed   = $B8  ;   Wall kick Y speed.
!NoMoveTime   = $10  ;   Frames in which the player won't be able to move after the wallkick.

;   You don't want to touch from here onwards.
!addr = $0000
if read1($00FFD5) == $23
       sa1rom
       !addr = $6000
endif

org $00CCB6
backup77:
       LDA $77
       STA !RAM_77Backup|!addr
org $00C599
       dw backup77

org $00A2D8
       autoclean JSL skWalljump

org $00F875
       autoclean JML freeScroll    ;   code by Noobish Noobsicle

freedata

;==================================

freeScroll:
       LDX $1404|!addr
       BNE +
       LDA !RAM_TimeInWall|!addr
       AND #$00FF
       BNE +
       JML $00F87A
+      JML $00F881

;==================================

kickXSpeeds:
       db !KickXSpeed,!KickXSpeed^$FF+1

skWalljump:
       ; lda $1337
       LDA !RAM_KickFlags|!addr
       BPL .notDisable
       LDY !RAM_DisableMovement|!addr
       DEY
       STY !RAM_DisableMovement|!addr
       BNE .disable
       LDA #$80
       TRB !RAM_KickFlags|!addr
       JML $00E2BD

.checkLastFrame
       LDA !RAM_77Backup|!addr
       AND #$03
       BEQ return
       LDX $76
       LDA.l wallSeparateSpeeds+2,x
       STA $7B
       STZ $7A
       if !ElevateSpeed != 0
              LDA #!ElevateSpeed
              STA $7D
       endif
returnClear:
       STZ !RAM_TimeInWall|!addr   ;   clear timer
return:
       JML $00E2BD                 ;   return (Mario's GFX routine)

skWalljump_disable:
       AND #$7F
       LSR #2
       TRB $15
       TRB $16
skWalljump_notDisable:
       LDA !RAM_TimeInWall|!addr   ;\
       BEQ +                       ; | decrement timer
       DEC !RAM_TimeInWall|!addr   ;/
       LDA $76
       INC
       EOR #$03
       TSB $15
       LDA $77                     ;\
       BEQ skWalljump_checkLastFrame

+      LDA $13EF|!addr             ;\  return if trying to climb while on ground
       BNE returnClear             ;/
       LDA $77
       BIT #$98                    ; | return if touching a ceiling, screen edges or inside a block
       BNE returnClear             ;/

       LDY !RAM_TimeInWall|!addr   ;\  no need to hold left/right
       BNE .dontCheckInput         ;/
       AND $15                     ;\  check that left is held
       BEQ returnClear             ;/
       LDY !RAM_KickFlags|!addr    ;
       BMI +                       ;
       ASL #2                      ;\  backup $77 again
       STA !RAM_KickFlags|!addr    ;/
+      LDA #!TimeToStick           ;\  store time to climb
       STA !RAM_TimeInWall|!addr   ;/

.dontCheckInput
       LDA !RAM_TimeInWall|!addr   ;\
       DEC                         ; | end wallclimbing if only 1 frame left
       BEQ .endWallclimb           ;/

       LDA $71                     ;\
       ORA $73                     ; |
       ORA $74                     ; |
       ORA $75                     ; | stop walljumping in case Mario is performing certain actions
       ORA $1407|!addr             ; | you can add your own checks, too
       ORA $140D|!addr             ; |
       ORA $1470|!addr             ; |
       ORA $187A|!addr             ; |
       BNE returnClear             ;/
       LDA $7E
       CMP #$09
       BCC returnClear
       CMP #$E8
       BCS returnClear

       LDY #$28                    ;\
       LDA $14                     ; |
       LSR #3                      ; |
       AND #$01                    ; | animation for the wallkick
       BNE +                       ; |
       INY                         ; |
+      STY $13E0|!addr             ;/

       LDY #!ClimbSpeed
       LDA $9D                     ;\  return if sprites locked
       BNE .return                 ;/
       LDA $16                     ;\
       PHP                         ; | set a different speed value if b is pressed
       BPL .storeSpeed             ;/
       LDY #!KickYSpeed
.storeSpeed
       STY $7D
       LDX $76
       LDA.l wallStaySpeeds,x
       PLP
       BPL +
       LDA #$01
       STA $1DF9|!addr
       STZ $77
       STZ !RAM_TimeInWall|!addr
       STZ !RAM_77Backup|!addr
       LDA #$80
       TSB !RAM_KickFlags|!addr
       LDA #!NoMoveTime
       STA !RAM_DisableMovement|!addr
       LDA #$0B
       STA $72
       LDA.l kickXSpeeds,x
+      STA $7B
.return
       JML $00E2BD                 ;   return (Mario's GFX routine)

.endWallclimb
       LDX $76
       LDA.l wallSeparateSpeeds,x
       STA $7B
       STZ $7A
       LDA #$F0
       STA $7D
       STZ $77
       LDA #!NoMoveTime
       STA !RAM_DisableMovement|!addr
       LDA #$80
       TSB !RAM_KickFlags|!addr
       JML $00E2BD                 ;   return (Mario's GFX routine)

wallStaySpeeds:
       db -$10,$10
wallSeparateSpeeds:
       db $18,-$18,$0A,-$0A

